#
# pushare - P2P file sharing system
#
# Copyright (c) 2004 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: Gateway.pm,v 1.6 2004/12/10 05:36:11 fuktommy Exp $
#

package PuShare::Gateway;
use strict;
use PuShare::Cache;
use PuShare::Config;
use PuShare::Interface;

my $cachedir = $PuShare::Config::cachedir;
my ($path, $port, $query_string);

sub printCache() {
	printHTMLHeader("cache - pushare");
	print "<h1>cache - pushare</h1><ul>\n";
	local $_;
	my %name = PuShare::Cache::list();
	my @file = ();
	foreach (glob "$cachedir/*.dat") {
		my $id = $_;
		$id =~ s|$cachedir/||;  $id =~ s|\.dat$||;
		push @file, $id;
	}
	@file = sort {$name{$a} cmp $name{$b}} @file;
	my $id;
	foreach $id (@file) {
		my $name = $name{$id};
		$name = "(null)" unless ($name);
		$name = escape($name);
		my $size = (-s "$cachedir/$id.dat");
		$size = int($size/100000)/10;
		my $xname = encode($name);
		print "<li>",
		      " <a href='/gw/ccmd?id=$id&amp;dist=1'>DIST</a>",
		      " <a href='/gw/ccmd?id=$id&amp;dele=1'>DELE</a>",
		      " <a href='/gw/ccmd?id=$id&amp;conv=1'>CONV</a>",
		      " - <a href=\"/gw/get/$id/$xname\">$name</a>($size MB)",
		      "</li>\n";

	}
	print "</ul>",
	      "<p><a href='/gw/ccmd?update=1'>UPDATE</a></p>",
	      "</body></html>\n";
}

sub printSearch() {
	printHTMLHeader("search - pushare");
	print "<h1>search - pushare</h1><ul>\n";
	my @search = ();
	if (open IN, $PuShare::Config::searchlist) {
		@search = <IN>;
		chomp @search;
		close IN;
	}
	local $_;
	foreach (@search) {
		my $x = escape($_);
		my $y = encode($_);
		print "<li>",
		      "<a href=\"/gw/scmd?delete=1&amp;exp=$y\">DELETE</a>",
		      " - $x",
		      "</li>\n";
	}
	print "</ul>",
	      "<form method='get' action='/gw/scmd'><p>",
	      "<input name='exp' size='30' tabindex='4' accesskey='e' value='' />",
	      "<input type='submit' value='ADD' name='add' tabindex='5' accesskey='a' />",
	      "</p></form>",
	      '<p>Ex: $arg{name}=~/jpg/i &amp;&amp; $arg{size}&lt;10*1024*1024</p>',
	      "</body></html>\n";
}

sub getFile($$) {
	my($id, $name) = @_;
	my $suffix = "";
	if ($name =~ /\.([^.]+)$/) {
		$suffix = lc $1;
	}
	my $type = "application/octet-stream";
	if ($PuShare::Config::mimeType{$suffix}) {
		$type = $PuShare::Config::mimeType{$suffix};
	}
	unless (open IN, "$cachedir/$id.dat") {
		print404();
		return;
	}
	printHeader("200 OK");
	print "Content-Type: $type\r\n",
	      "Content-Lenget: ", (-s "$cachedir/$id.dat"), "\r\n",
	      "\r\n";
	local $_;
	while (<IN>) {
		print;
	}
	close IN;
}

sub command_cache() {
	my %arg = args($query_string);
	if ($arg{dele} && $arg{id}) {
		PuShare::Cache::delete($arg{id});
		print302("/gw/cache");
	} elsif ($arg{conv} && $arg{id}) {
		PuShare::Cache::convert($arg{id});
		print302("/gw/cache");
	} elsif ($arg{dist} && $arg{id}) {
		PuShare::Util::addDist(time, $arg{id});
		my $name = PuShare::Cache::name($arg{id});
		$name = encode($name);
		PuShare::NodeList::relayDist("port=$port&id=$arg{id}&name=$name");
		print302("/gw/cache");
	} elsif ($arg{update}) {
		PuShare::Cache::update();
		print302("/gw/cache");
	} else {
		print404();
	}
}

sub command_search() {
	my %arg = args($query_string);
	if ($arg{delete} && $arg{exp}) {
		PuShare::Search::delete($arg{exp});
		print302("/gw/search");
	} elsif ($arg{add} && $arg{exp}) {
		PuShare::Search::add($arg{exp});
		print302("/gw/search");
	} else {
		print404();
	}
}

#------------------------------------------------------------------#
sub main() {
	$path = $main::path;
	$port = $main::port;
	$query_string = $main::query_string;

	if ($main::remote_addr !~ /$PuShare::Config::adminAddr/) {
		print404();

	} elsif (($path eq "/") || ($path eq "/gw") ||
		 ($path eq "/gw/") || ($path eq "/gw/cache")) {
		printCache();

	} elsif ($path eq "/gw/search") {
		printSearch();

	} elsif ($path =~ m|^/gw/get/([0-9a-f]{32})/([0-9A-Za-z.%_]+)$|) {
		my($id, $name) = ($1, $2);
		getFile($id, $name);

	} elsif ($path eq "/gw/ccmd") {
		command_cache();

	} elsif ($path eq "/gw/scmd") {
		command_search();

	} else {
		print404();
	}
}

1;
